/*
Solution Code for Lab 3
University of Michigan
EECS-461, Embedded Systems
Queued Analog-to-Digital Converter Module-64 (QADC64)
Read Chapter 13.12 in MPC555 User's Manual
Created: 7/16/02 by Eric Castle
*/


#include "qadc64.h"
#include "mpc555.h"
//#include "assert.h"


// Initialize the qadc64 module
void qadc64Init()
{ 
  /*initialize QADC64INT register to disable interrupts*/
  QADC_A.QADC64INT.B.IRL1 = 0;
  QADC_A.QADC64INT.B.IRL1 = 0;

  // Set the MCR to allow for access to other registers
  QADC_A.QADC64MCR.B.SUPV = 0; // Allow easy access (1 would restrict to supervisor access)

  // Settings for QACR0
  QADC_A.QACR0.B.PSL = 7;  // Listed in the lab
  QADC_A.QACR0.B.PSH = 15; 
  QADC_A.QACR0.B.MUX = 0;  // Set to internal multiplexing

  // Settings for the QACR1 - sets queue 1 to software-initiated continuous scan mode
  QADC_A.QACR1.B.CIE1 = 0; // Disable interrupts
  QADC_A.QACR1.B.PIE1 = 0; // Disable pause for interrupts
  QADC_A.QACR1.B.MQ1 = 17; // Software triggered continuous scan mode
  QADC_A.QACR1.B.SSE1 = 1; // Start conversions
  
  // Settings for the QACR2 - sets queue 2 according to the specs
  QADC_A.QACR2.B.MQ2 = 1; // Software triggered single-scan mode
  QADC_A.QACR2.B.CIE2 = 0; // Disable completion interrupts
  QADC_A.QACR2.B.PIE2 = 0; // Disable pause for interrupts (probably not needed)
  QADC_A.QACR2.B.BQ2 = 16;  // Set the queue to begin conversions at 16

  // Setup conversions - fills CCW table
  fillCCWTable();

}


/* 
Read an analog input from queue 1 in the qadc64 module.  Since it is constantly
performing conversions, all that needs to be done is read the value from the result table.
*/
int qadc64ReadQ1(int module, int channel)
{
  int value;

  // Make sure module is 0
//  assert(!module);

  value = QADC_A.RJURR[channel].R;

  return value;
}


//Read an analog input from the qadc64 module Queue 2
//This function must initiate a scan and wait for the conversion to finish, before
//reading the result
int qadc64ReadQ2(int module, int channel)
{
  int value;

  //Both inputs should be 0
//  assert( !(module || channel));

  QADC_A.QACR2.B.SSE2 = 1; // Start conversion

  // Wait for it to finish
  while(QADC_A.QASR0.B.CF2 == 0)
    {} 

  value = QADC_A.RJURR[channel].R;
    
  return value;
}


// Configures the CCW Table so as to do the proper conversions.  
void fillCCWTable()
{
  QADC_A.CCW[0].B.P    = 0;
  QADC_A.CCW[0].B.BYP  = 0;
  QADC_A.CCW[0].B.IST  = 1;
  QADC_A.CCW[0].B.CHAN = 0; /*cause a read from chan0, place it into index0*/
    
  QADC_A.CCW[1].R = 63; /*end of command chain for queue 1*/

  //NOTE: Without an external MUX, channels 16-31 (among others) do not exist
  //      These are here because they were part of an example file.
  
  QADC_A.CCW[16].B.P    = 0; /*rest is bogus */
  QADC_A.CCW[16].B.BYP  = 0;
  QADC_A.CCW[16].B.IST  = 1;
  QADC_A.CCW[16].B.CHAN = 0;

  QADC_A.CCW[17].B.P    = 0;
  QADC_A.CCW[17].B.BYP  = 0;
  QADC_A.CCW[17].B.IST  = 1;
  QADC_A.CCW[17].B.CHAN = 1;

  QADC_A.CCW[18].B.P    = 0;
  QADC_A.CCW[18].B.BYP  = 0;
  QADC_A.CCW[18].B.IST  = 1;
  QADC_A.CCW[18].B.CHAN = 2;

  QADC_A.CCW[19].B.P    = 0;
  QADC_A.CCW[19].B.BYP  = 0;
  QADC_A.CCW[19].B.IST  = 1;
  QADC_A.CCW[19].B.CHAN = 3;

  QADC_A.CCW[20].B.P    = 0;
  QADC_A.CCW[20].B.BYP  = 0;
  QADC_A.CCW[20].B.IST  = 1;
  QADC_A.CCW[20].B.CHAN = 52;

  QADC_A.CCW[21].B.P    = 0;
  QADC_A.CCW[21].B.BYP  = 0;
  QADC_A.CCW[21].B.IST  = 1;
  QADC_A.CCW[21].B.CHAN = 53;

  QADC_A.CCW[22].B.P    = 0;
  QADC_A.CCW[22].B.BYP  = 0;
  QADC_A.CCW[22].B.IST  = 1;
  QADC_A.CCW[22].B.CHAN = 54;

  QADC_A.CCW[23].B.P    = 0;
  QADC_A.CCW[23].B.BYP  = 0;
  QADC_A.CCW[23].B.IST  = 1;
  QADC_A.CCW[23].B.CHAN = 55;

  QADC_A.CCW[24].R  = 63; // End of queue

}

